/*
 * Copyright (C) 2004 Andre Noll <noll@mathematik.tu-darmstadt.de>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"
#include "config.h"
#include "pob-ogg.cmdline.h"

#define OGG_BUFFSIZE 40000

struct gengetopt_args_info args;

/*
 * we always log to stderr
 */
void clog(int ll, char* fmt,...)
{
	va_list argp;

	/* ignore log message if loglevel is not high enough */
	if (ll < args.loglevel_arg)
		return;
	va_start(argp, fmt);
	vfprintf(stderr, fmt, argp);
	va_end(argp);
}

void get_options(int argc, char * const *argv, char **config_file)
{
	char *home = getenv("HOME");
	static char default_config_file[_POSIX_PATH_MAX] = "";
	struct stat statbuf;
	int ret;

	cmdline_parser(argc, argv, &args);
	if (!args.config_file_given) {
		sprintf(default_config_file, "%s/.paraslash/pob-ogg.conf",
			home);
		*config_file = default_config_file;
	} else
		*config_file = args.config_file_arg;
	ret = stat(*config_file, &statbuf);
	if (ret && args.config_file_given) {
		clog(EMERG, "Can't stat config file %s\n",
			args.config_file_arg);
		exit(EXIT_FAILURE);
	}
	if (!ret)
		cmdline_parser_configfile(*config_file, &args, 0, 0, 0);
}

ssize_t find_ogg_offset(char *buf, size_t len)
{
	char *p = buf;
	size_t pos = 0;

	clog(DEBUG, "%s: Finding ogg offset (buffer length = %d)\n", __func__,
		len);
	while (p + pos + 4 <= buf + len) {
		pos++;
		if (*p++ != 'O')
			continue;
		pos++;
		if (*p++ != 'g')
			continue;
		pos++;
		if (*p++ != 'g')
			continue;
		pos++;
		if (*p++ != 'S')
			continue;
		return pos - 4;
	}
	clog(DEBUG, "%s: OggS signature not found\n", __func__);
	return -1;
}

/*
 * MAIN
 */
int main(int argc, char * const *argv) {

	int sockfd, ogg_pos, ret;
	ssize_t numbytes = 0;
	struct hostent *he;
	struct sockaddr_in their_addr;
	char buf[OGG_BUFFSIZE];
	char header[STRINGSIZE];
	char *config_file;

	get_options(argc, argv, &config_file);
	if (args.loglevel_arg <= NOTICE)
		cmdline_parser_print_version();
	clog(INFO, "%s:"
		"Current loglevel: %d\n"
		"Config file: %s\n",
		__func__,
		args.loglevel_arg,
		config_file
	);
	
	/* get the host info */
	clog(NOTICE, "%s: Getting host info of %s\n", __func__,
		args.hostname_arg);
	if (!(he = get_host_info(args.hostname_arg)))
		exit(EXIT_FAILURE);
	/* get new socket */
	if ((sockfd = get_socket()) < 0)
		exit(EXIT_FAILURE);
	/* init their_addr */
	init_sockaddr(&their_addr, args.server_port_arg, he);
	/* Connect */
	clog(NOTICE, "%s: Connecting to %s...\n", __func__,
		args.hostname_arg);
	if (do_connect(sockfd, &their_addr) < 0) {
		clog(EMERG, "%s: Connect failed (%s)\n", __func__,
			strerror(errno));
		exit(EXIT_FAILURE);
	}
	clog(DEBUG, "%s: Sending request\n", __func__);
	if (send_buffer(sockfd, "GET /////////////////\n") < 0) {
		clog(EMERG, "%s: Error (%s)\n", __func__, 
			strerror(errno));
		exit(EXIT_FAILURE);
	}
	clog(DEBUG, "%s: receiving header\n", __func__);
	/* Receive header */
	if ((numbytes = recv_buffer(sockfd, header, STRINGSIZE - 1)) < 0)
		exit(EXIT_FAILURE);
	/* 
	 * the header may contain some HHTP message which confuses
	 * decoders
	 */
	ogg_pos = find_ogg_offset(header, numbytes);
	if (ogg_pos < 0) /* ignore the whole thing, it's only HTML */
		clog(DEBUG, "%s: No ogg signature found, skipping\n", __func__);
	else {
		clog(INFO, "%s: skipping %d bytes\n", __func__, ogg_pos);
		write(STDOUT_FILENO, header + ogg_pos, numbytes - ogg_pos);
	}
	if (send_buffer(sockfd, header) < 0)
		exit(EXIT_FAILURE);
	/* fill our buffer halfways */
	numbytes = 0;
#if 0
	clog(INFO, "%s: prebuffering %d bytes\n", __func__, OGG_BUFFSIZE / 2);
	while (numbytes < OGG_BUFFSIZE / 2) { 
		ret = recv_buffer(sockfd, buf + numbytes, OGG_BUFFSIZE / 2);
		if (ret <= 0)
			exit(EXIT_FAILURE);
		numbytes += ret;
		if (send_buffer(sockfd, header) < 0)
			exit(EXIT_FAILURE);
	}
	write(STDOUT_FILENO, buf, numbytes);
#endif 
	/* write server output to stdout */
	while ((numbytes = recv_buffer(sockfd, buf, STRINGSIZE)) > 0) {
		ret = write(STDOUT_FILENO, buf, numbytes);
		if (ret < 0)
			exit(EXIT_FAILURE);
		if (!ret)
			clog(DEBUG, "%s: short write\n", __func__);
		if (send_buffer(sockfd, header) < 0)
			exit(EXIT_FAILURE);
	}
	if (numbytes < 0) {
		clog(ERROR, "%s: Read %i (%s)\n", __func__, numbytes, strerror(errno));
		exit(EXIT_FAILURE);
	}
	if (!numbytes)
		clog(NOTICE, "%s: Connection closed by peer\n", __func__);
	close(sockfd);
	return 0;
}
